@@ -13,11 +13,12 @@ import os
 import warnings
 warnings.filterwarnings("ignore", category=SyntaxWarning)
 
-__version__ = "1.0.20"
+__version__ = "1.1.3"
 __auto_init__ = 12345
 
 global_options = optparse.OptionParser(
-    usage="aos COMMAND [ARGS]"
+    usage="aos COMMAND [ARGS]",
+    description="For more information about aos commands list, please see `aos help`.\t\tFor a specific command, please see `aos help <command>`."
 )
 
 
@@ -90,6 +91,7 @@ class YocCommand:
                     return 1
             else:
                 put_string("aos: '%s' is not a aos command.  See 'aos help'." % name, file=sys.stderr)
+                check_remote_version()
                 return 1
 
         try:
@@ -106,6 +108,7 @@ class YocCommand:
             result = cmd.Execute(copts, cargs)
         except Exception as e:
             put_string("AosCommand error:", e)
+            check_remote_version()
             result = 1
             pass
         return result
