@@ -141,13 +141,19 @@ class Component:
                 branch = 'master'
             else:
                 put_string("%-16s(%s), %s %s ..." % (self.name, branch, pull_str, self.repo_url))
-            if pull_str == "clone" or branch in tags:
-                # case 2: download a new git 
+            if pull_str == "clone":
+                # case 2: new git, download a new git
+                # case 5: prior branch has no package.yaml or README.md file, and switch to different branch
+                git.pull(branch, None)
+                git.CheckoutBranch(branch)
+            elif branch in tags:
                 # case 3: switch to different tag
                 git.pull(branch, None)
             else:
                 # case 4: switch to different branch
                 git.CheckoutBranch(branch)
+            if not self.check_file_integrity():
+                put_string("There is no `README.md` or `package.yaml` file found in %s component,please add it." % self.name)
 
         elif self.version in self.historyVersion.keys():
             zip_url = self.historyVersion[self.version]
@@ -160,7 +166,7 @@ class Component:
 
     def upload(self, repo_base_url=''):
         if not self.check_file_integrity():
-            put_string("There is no `README.md` file found in %s component,please add it first." % self.name)
+            put_string("There is no `README.md` or `package.yaml` file found in %s component,please add it first." % self.name)
             exit(-1)
         repo_url = "%s/%s.git" % (self.conf.repo, self.name)
         if self.conf.repo.endswith('.git'):
@@ -180,7 +186,7 @@ class Component:
             put_string("Component `%s` upload fail." % self.name)
 
         try:
-            shutil.rmtree('/tmp/' + self.name)
+            rmtree_enhanced('/tmp/' + self.name)
         except Exception as e:
             put_string("%s" % str(e))
             exit(-1)
@@ -257,6 +263,11 @@ class Component:
             if type(d) == dict:
                 self.depends.append(d)
 
+        # add component version in defconfig
+        ver_key = "%s_SW_VERSION" % self.name.upper()
+        self.defconfig[ver_key] = "%s_%s" % (self.name, self.version)
+        if self.type == "solution" and "SW_VERSION" in self.defconfig.keys():
+            self.defconfig[ver_key] = "%s_%s_%s" % (self.name, self.version, self.defconfig["SW_VERSION"])
         self.loaded = True
         return True
 
@@ -447,7 +458,7 @@ class Component:
             self.path = new_path
             self.name = new_name
             if os.path.exists(os.path.join(self.path, '.git')):
-                shutil.rmtree(os.path.join(self.path, '.git')) 
+                rmtree_enhanced(os.path.join(self.path, '.git')) 
 
             return True
         except Exception as e:
@@ -516,12 +527,15 @@ class ComponentGroup(list):
 
                     c = self.components.get(name)
                     if c:
+                        append_flag = False
+                        # cond_break: get mandatory component only or not
                         if cond_break:
                             v = cond.split('?')
                             if len(v) > 1:
                                 continue
                         if c not in depends:
                             depends.append(c)
+                            append_flag = True
                         # print depend chain
                         if print_depends:
                             if len(v) > 1:
@@ -533,7 +547,10 @@ class ComponentGroup(list):
 
                         if component not in c.depends_on:
                             c.depends_on.append(component)
-                        _check_depend(c)
+                        # if c appended to depends in this loop, check it's depends.
+                        # otherwise, c is already in depends, just ignore it.
+                        if append_flag:
+                            _check_depend(c)
 
         depends = ComponentGroup()
 
