@@ -370,8 +370,8 @@ class Solution:
 
         update_variables()
 
-        def check_variables(values=[]):
-            # check and condition: [v1, v2, v3...]
+        def check_variable(values=[]):
+            # check and condition: [v1, v2, v3...], AND operation
             for v in values:
                 if(v.startswith('!')):
                     a = self.variables.get(v[1:])
@@ -384,6 +384,14 @@ class Solution:
 
             return True
 
+        def check_variables(values=[]):
+            # check and condition: [['aa'], ['bb', 'cc'], ['dd', 'ee']] or [['ff']], OR operation
+            for v in values:
+                if check_variable(v):
+                    return True
+
+            return False
+
         for cc in self.cond_components:
             if check_variables(cc.conditions):
                 self.components.add(cc)
