@@ -252,50 +252,148 @@ class YoC:
                     depends_on.append(c)
         return depends_on
 
-    def check_conditions(self, components=[], exclude_names=[]):
-        cmpts = ComponentGroup()
-        for comp in components:
+    def get_comp_mandatory_depends(self, comps_list, parent_names, exclude_names):
+        """ Get comp mandatory depends from comps_list """
+        depends = []
+        for comp in comps_list:
+            if comp.name in parent_names:
+                for dep_dict in comp.depends:
+                    name = list(dep_dict.keys())[0]
+                    cond = list(dep_dict.values())[0]
+                    if (name not in exclude_names) and (cond.find('?') < 0):
+                        depends.append(name)
+                        # print("add mandatory depend: %s for %s." % (name, comp.name))
+
+        if depends:
+            # ignore the component which is already in parent and it's depends
+            exclude_names = exclude_names + parent_names + depends
+            exclude_names = list(set(exclude_names))
+            depends += self.get_comp_mandatory_depends(comps_list, depends, exclude_names)
+
+        return list(set(depends))
+
+    def get_comp_optional_depends(self, comps_list, parent_names, exclude_names):
+        """ Get comp optional depends from comps_list """
+        depends = []
+        """ parent_names are mandatory components got by get_comp_mandatory_depends,
+        here is to find all optional dependencies for parent_names"""
+        for comp in comps_list:
+            if comp.name in parent_names:
+                for dep_dict in comp.depends:
+                    name = list(dep_dict.keys())[0]
+                    cond = list(dep_dict.values())[0]
+                    if (name not in exclude_names) and (name not in parent_names) and (cond.find('?') >= 0):
+                        v = cond.split('?')
+                        if len(v) > 1:
+                            x = re.search('<(.+?)>', v[1], re.M | re.I)
+                            if x:
+                                cond = x.group(1)
+                                tmp = {}
+                                tmp["comp_name"] = name
+                                tmp["count"] = 0
+                                tmp["condition"] = [cond]
+                                depends.append(tmp)
+                                # print("add depend:%s with cond:%s for %s" % (name, cond, comp.name))
+
+        merge_depends = []
+        if depends:
+            depends += self.get_comp_optional_depends_r(comps_list, depends, list(set(parent_names).union(set(exclude_names))))
+            merge_depends = self.merge_comp_optional_depends(depends)
+
+        return merge_depends
+
+    def get_comp_optional_depends_r(self, comps_list, parent_names, exclude_names):
+        """ Get comp optional depends recursively from comp index """
+        depends = []
+        """ comps are optional dependency list from last layer """
+        for comp in comps_list:
+            parent_found = {}
+            for parent_name in parent_names:
+                if comp.name == parent_name["comp_name"]:
+                    parent_found = parent_name
+                    break
+            if not parent_found:
+                continue
+            # if recursive level is >= 64, there should be dead loop, ignore this comp forcely
+            if parent_found["count"] >=64:
+                continue
             for dep_dict in comp.depends:
                 name = list(dep_dict.keys())[0]
                 cond = list(dep_dict.values())[0]
-                v = cond.split('?')
-                if len(v) > 1:
-                    x = re.search('<(.+?)>', v[1], re.M | re.I)
-                    if x:
-                        cond = x.group(1)
-                        c = self.components.get(name)
-                        if c:
-                            c.conditions.append(cond)
-                            cmpts.add(c)
-                            # print("comp %s has depends %s with cond %s" % (comp.name, name, cond))
-        # 添加所有依赖于此组件的条件
-        for comp in cmpts:
-            # print("ck1##%s" % comp.name)
-            for c in comp.depends_on:
-                if c in cmpts:
-                    for cond1 in c.conditions:
-                        if cond1 not in comp.conditions:
-                            comp.conditions.append(cond1)
-                            # print("comp %s inherit cond %s from parent %s" % (comp.name, cond1, c.name))
-        # 添加所有依赖的组件的条件
-        for comp in cmpts:
-            depends, _ = self.components.get_depend(comp, exclude_names)
-            for dep in depends:
-                # print("ck2##%s" % dep.name)
-                for cond1 in comp.conditions:
-                    if cond1 not in dep.conditions:
-                        dep.conditions.append(cond1)
-                        # print("depend %s inherit cond %s from comp %s" % (dep.name, cond1, comp.name))
-        # 去除强依赖的组件的条件数组
+                """ get mandatory dependency list for this optional component """
+                if (name not in exclude_names) and (cond.find('?') < 0):
+                    """ add to the list with the inherrited dependency"""
+                    tmp = {}
+                    tmp["comp_name"] = name
+                    tmp["count"] = parent_found["count"] + 1
+                    tmp["condition"] = parent_found["condition"]
+                    depends.append(tmp)
+                    # print(tmp["count"])
+                    # print("add depend:%s with inherrited cond:%s for %s" % (name, parent_found["condition"], comp.name))
+                """ get optional dependency list for this optional component """
+                if (name not in exclude_names) and (cond.find('?') >= 0):
+                    v = cond.split('?')
+                    if len(v) > 1:
+                        x = re.search('<(.+?)>', v[1], re.M | re.I)
+                        if x:
+                            """ add to the list with (the inherrited dependency && this condition) """
+                            cond = x.group(1)
+                            tmp = {}
+                            tmp["comp_name"] = name
+                            tmp["count"] = parent_found["count"] + 1
+                            if cond not in parent_found["condition"]:
+                                tmp["condition"] = parent_found["condition"] + [cond]
+                            else:
+                                tmp["condition"] = parent_found["condition"]
+                            depends.append(tmp)
+                            # print(tmp["count"])
+                            # print("add depend:%s with inherrited cond:%s and append cond: %s for %s" % (name, parent_found["condition"], cond, comp.name))
+
+        if depends:
+            depends += self.get_comp_optional_depends_r(comps_list, depends, exclude_names)
+        return depends
+
+    def merge_comp_optional_depends(self, optional_deps):
+        """ merge the condition for the dependency of same component name """
+        merge_depends = []
+        if optional_deps:
+            optional_deps.sort(key=lambda x: x["comp_name"])
+            last_dep = ""
+            for dep in optional_deps:
+                # print("optional dependency is", dep)
+                if dep["comp_name"] != last_dep:
+                    """ new deps """
+                    tmp = {}
+                    tmp["comp_name"] = dep["comp_name"]
+                    tmp["condition"] = []
+                    tmp["condition"].append(dep["condition"])
+                    last_dep = dep["comp_name"]
+                    merge_depends.append(tmp)
+                else:
+                    """ deps with the prio one """
+                    duplicated = False
+                    for cond in merge_depends[-1]["condition"]:
+                        if cond == dep["condition"]:
+                            duplicated = True
+                            break
+                    if not duplicated:
+                        merge_depends[-1]["condition"].append(dep["condition"])
+
+        return merge_depends
+
+    def check_conditions(self, components=[], parent_names=[], exclude_names=[]):
+        mandatory_deps = self.get_comp_mandatory_depends(components, parent_names, exclude_names)
+        mandatory_deps += parent_names
+        optional_deps = self.get_comp_optional_depends(components, mandatory_deps, exclude_names)
+
+        # 将条件依赖放入到组件的conditions变量中
         for comp in components:
-            if comp not in cmpts and (len(comp.conditions) == 0):
-                depends, _ = self.components.get_depend(comp, exclude_names, cond_break=True)
-                for dep in depends:
-                    dep.conditions = []
-                    # print("comp %s has depend %s directly, clear %s condition" %(comp.name, dep.name, dep.name))
+            for opt_dep in optional_deps:
+                if comp.name == opt_dep["comp_name"]:
+                    comp.conditions = opt_dep["condition"]
+                    break
 
-
-    def getSolution(self, board_name=None, compiler='gcc', print_depends=False):
+    def getSolution(self, board_name=None, compiler='gcc', print_depends=False, exit_if_lost=True):
         for component in self.components:
             if component.path == os.getcwd():
                 component.load_package()
@@ -318,25 +416,31 @@ class YoC:
 
                 components, lost = self.components.get_depend(component, exclude_names, print_depends=print_depends)
                 if lost:
-                    self.auto_install_missing_comp(lost)
-                    put_string("Please try it again.")
-                    exit(-1)
+                    ret = self.auto_install_missing_comp(lost)
+                    if (ret != 0) or exit_if_lost:
+                        put_string("Please make sure ", lost, "are valid, and try it again.")
+                        exit(-1)
+                    else:
+                        return {}
                 components.add(component)
                 # depends and condition
-                self.check_conditions(components, exclude_names)
+                parent_names = [component.name, component.hw_info.board_name]
+                self.check_conditions(components, parent_names, exclude_names)
                 solution = Solution(components, compiler)
 
                 return solution
 
     def auto_install_missing_comp(self, lost_comps):
         put_string("Try to install missing component...")
+        ret = 0
         for comp_dict in lost_comps:
             name = list(comp_dict.keys())[0]
             cond = list(comp_dict.values())[0]
             v = cond.split('?')
             if len(v) > 1:
                 cond = v[0]
-            os.system("aos install %s -b %s" %(name, cond))
+            ret = ret | (os.system("aos-tools install %s -b %s" %(name, cond)) >> 8)
+        return ret
 
     def check_cmpt_download(self, name, update=True, force=False):
         if self.components.get(name) and not force:
@@ -385,7 +489,7 @@ class YoC:
                             del p.depends_on[name]
                         self.remove_component(n)
 
-                shutil.rmtree(component.path)
+                rmtree_enhanced(component.path)
                 self.components.remove(component)
                 return True
             else:
